@@ -25,15 +25,16 @@
 #include <glade/glade.h>
 
 #include <signal.h>
+#include <ctype.h>
 
 #include "bug-buddy.h"
+#include "util.h"
+#include "distro-redhat.h"
+#include "distro-debian.h"
 
 /* libglade callbacks */
 gboolean on_nature_page_next  (GtkWidget *, GtkWidget *);
-gboolean on_attach_page_next  (GtkWidget *, GtkWidget *);
-gboolean on_core_page_back    (GtkWidget *, GtkWidget *);
 gboolean on_less_page_prepare (GtkWidget *, GtkWidget *);
-gboolean on_less_page_back    (GtkWidget *, GtkWidget *);
 gboolean on_less_page_next    (GtkWidget *, GtkWidget *);
 gboolean on_misc_page_back    (GtkWidget *, GtkWidget *);
 gboolean on_the_druid_cancel  (GtkWidget *);
@@ -49,10 +50,16 @@ void on_version_apply_clicked (GtkButton *button, gpointer data);
 void on_file_radio_toggled    (GtkWidget *radio, gpointer data);
 gboolean on_action_page_next  (GtkWidget *page, GtkWidget *druid);
 gboolean on_action_page_back  (GtkWidget *page, GtkWidget *druid);
-GtkWidget * make_anim (gchar *widget_name, gchar *string1, 
-		       gchar *string2, gint int1, gint int2);
-GtkWidget * make_pixmap_button (gchar *widget_name, gchar *string1, 
-				gchar *string2, gint int1, gint int2);
+GtkWidget *make_anim (gchar *widget_name, gchar *string1, 
+		      gchar *string2, gint int1, gint int2);
+GtkWidget *make_pixmap_button (gchar *widget_name, gchar *string1, 
+			       gchar *string2, gint int1, gint int2);
+void on_newreport_radio_toggled (GtkWidget *w, gpointer data);
+void on_existing_radio_toggled (GtkWidget *w, gpointer data);
+
+
+
+#define LINE_WIDTH 72
 
 extern const char *packages[];
 
@@ -89,44 +96,38 @@ struct {
 
 DruidData druid_data;
 
-static ListData list_data[] = {
-	{ N_("System"), { "uname -a" } },
-	{ N_("Distribution"),
-	  { "( [ -f /etc/slackware-version ] && "
-	    "  echo -n \"Slackware \" && cat /etc/slackware-version) || "
-	    "( [ -f /etc/slackware-version ] && "
-	    "  echo -n \"Debian \" && cat /etc/debian_version) || "
-	    "( [ -f /etc/redhat-release ] && cat /etc/redhat-release) || "
-	    "( [ -f /etc/SuSE-release ]   && head -1 /etc/SuSE-release) ||"
-	    "echo \"\"" } },
-	{ N_("C library"), { "rpm -q glibc",  "rpm -q libc" } },
-	{ N_("C Compiler"), { "gcc --version", "cc -V" } },
-	{ N_("glib"), { "glib-config --version", "rpm -q glib" } },
-	{ N_("GTK+"), { "gtk-config --version", "rpm -q gtk+" } },
-	{ N_("ORBit"), { "orbit-config --version", "rpm -q ORBit" } },
-	{ N_("gnome-libs"), { "gnome-config --version", "rpm -q gnome-libs" } },
-	{ N_("gnome-core"), { "gnome-config --modversion applets "
-			      "| grep -v gnome-libs "
-			      "| sed -e 's^applets-^gnome-core ^g'", 
-			      "rpm -q gnome-core" } },
+static Distribution distros[] = {
+	{ "Slackware", "/etc/slackware-version", &debian_phy },
+	{ "Debian",    "/etc/debian_version",    &debian_phy },
+	{ "Red Hat",   "/etc/redhat-release",    &redhat_phy },
+	{ "SuSE",      "/etc/SuSE-release",      &redhat_phy },
+	{ "Mandrake",  "/etc/mandrake-release",  &redhat_phy },
+	{ NULL }
+};
+
+static Package package[] = {
+	{ N_("System"),    "uname -a" },
+	{ N_("C library"),  NULL,                    "glibc", "libc6", },
+	{ N_("C compiler"), "gcc --version",          NULL,    NULL, "cc -V", },
+	{ N_("glib"),       "glib-config --version",  "glib",  "libglib1.2" },
+	{ N_("GTK+"),       "gtk-config --version",   "gtk+",  "libgtk1.2" },
+	{ N_("ORBit"),      "orbit-config --version", "ORBit", "liborbit0" },
+	{ N_("gnome-libs"), "gnome-config --version", "gnome-libs", "gnome-libs-data" },
+	{ N_("gnome-core"),  "gnome-config --modversion applets "
+	                     "| grep -v gnome-libs "
+	                     "| sed -e 's^applets-^gnome-core ^g'",
+	  "gnome-core", "gnome-core" },
 	{ NULL }
 };
 
 static const struct poptOption options[] = {
-	{ "name",        0, POPT_ARG_STRING, &popt_data.name,        
-	  0, N_("Contact's name"),          N_("NAME") },
-	{ "email",       0, POPT_ARG_STRING, &popt_data.email,       
-	  0, N_("Contact's email address"), N_("EMAIL") },
-	{ "package",     0, POPT_ARG_STRING, &popt_data.package,     
-	  0, N_("Package of the program"),  N_("PACKAGE") },
-	{ "package-ver", 0, POPT_ARG_STRING, &popt_data.package_ver, 
-	  0, N_("Version of the package"),  N_("VERSION") },
-	{ "appname",     0, POPT_ARG_STRING, &popt_data.app_file,    
-	  0, N_("Crashed file name"),       N_("FILE") },
-	{ "pid",         0, POPT_ARG_STRING, &popt_data.pid,         
-	  0, N_("PID of crashed app"),      N_("PID") },
-	{ "core",        0, POPT_ARG_STRING, &popt_data.core_file,   
-	  0, N_("core file from app"),      N_("FILE") },
+	{ "name",        0, POPT_ARG_STRING, &popt_data.name,        0, N_("Contact's name"),          N_("NAME") },
+	{ "email",       0, POPT_ARG_STRING, &popt_data.email,       0, N_("Contact's email address"), N_("EMAIL") },
+	{ "package",     0, POPT_ARG_STRING, &popt_data.package,     0, N_("Package of the program"),  N_("PACKAGE") },
+	{ "package-ver", 0, POPT_ARG_STRING, &popt_data.package_ver, 0, N_("Version of the package"),  N_("VERSION") },
+	{ "appname",     0, POPT_ARG_STRING, &popt_data.app_file,    0, N_("Crashed file name"),       N_("FILE") },
+	{ "pid",         0, POPT_ARG_STRING, &popt_data.pid,         0, N_("PID of crashed app"),      N_("PID") },
+	{ "core",        0, POPT_ARG_STRING, &popt_data.core_file,   0, N_("core file from app"),      N_("FILE") },
 	{ NULL } 
 };
 
@@ -150,9 +151,10 @@ static void
 save_config ()
 {
 
-	save_entry ("name_entry",  "name_entry2",  "/bug-buddy/last/name");
-	save_entry ("email_entry", "email_entry2", "/bug-buddy/last/email");
-	save_entry ("file_entry",  "file_entry2",  "/bug-buddy/last/bugfile");
+	save_entry ("name_entry",   "name_entry2",   "/bug-buddy/last/name");
+	save_entry ("email_entry",  "email_entry2",  "/bug-buddy/last/email");
+	save_entry ("file_entry",   "file_entry2",   "/bug-buddy/last/bugfile");
+	save_entry ("mailer_entry", "mailer_entry2", "/bug-buddy/last/mailer");
 	
 	gnome_config_set_int ("/bug-buddy/last/submittype", 
 			      druid_data.submit_type);
@@ -183,6 +185,8 @@ load_entry (const char *name, const char *name2,
 static void
 load_config ()
 {
+	char *sendmail;
+
 	load_entry ("name_entry", "name_entry2",
 		    "/bug-buddy/last/name", g_get_real_name ());
 
@@ -192,18 +196,47 @@ load_config ()
 	load_entry ("file_entry", "file_entry2",
 		    "/bug-buddy/last/bugfile", NULL);
 
+	sendmail = gnome_is_program_in_path ("sendmail");
+	if (!sendmail) {
+		if (g_file_exists ("/usr/sbin/sendmail"))
+			sendmail = g_strdup ("/usr/sbin/sendmail");
+		else if (g_file_exists ("/usr/lib/sendmail"))
+			sendmail = g_strdup ("/usr/lib/sendmail");
+	}
+	load_entry ("mailer_entry", "mailer_entry2",
+		    "/bug-buddy/last/mailer", sendmail);
+	g_free (sendmail);
+
 	druid_data.submit_type = 
-		gnome_config_get_int ("/bug-buddy/last/submittype=0");
-	
+		gnome_config_get_int ("/bug-buddy/last/submittype");
 }
 
 static gboolean
 update_crash_type (GtkWidget *w, gpointer data)
 {
 	CrashType new_type = GPOINTER_TO_INT (data);
-	
+	GtkWidget *segv, *core;
+
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)))
 		druid_data.crash_type = new_type;
+
+	segv = glade_xml_get_widget (druid_data.xml, "segv_table");
+	core = glade_xml_get_widget (druid_data.xml, "core_box");
+
+	switch (new_type) {
+	case CRASH_DIALOG:
+		gtk_widget_show (segv);
+		gtk_widget_hide (core);
+		break;
+	case CRASH_CORE:
+		gtk_widget_show (core);
+		gtk_widget_hide (segv);
+		break;
+	case CRASH_NONE:
+		gtk_widget_hide (core);
+		gtk_widget_hide (segv);
+		break;
+	}
 	
 	return FALSE;
 }
@@ -222,36 +255,11 @@ update_submit_type (GtkWidget *w, gpointer data)
 gboolean
 on_nature_page_next (GtkWidget *page, GtkWidget *druid)
 {
-	GtkWidget *newpage;
-	switch (druid_data.crash_type) {
-	case CRASH_DIALOG:
+	if (druid_data.crash_type != CRASH_NONE)
 		return FALSE;
-		break;
-	case CRASH_CORE:
-		newpage = druid_data.core;
-		break;
-	default:
-		newpage = druid_data.action;
-		break;
-	}
-	gnome_druid_set_page (GNOME_DRUID (druid),
-			      GNOME_DRUID_PAGE (newpage));
-	return TRUE;
-}
 
-gboolean
-on_attach_page_next (GtkWidget *page, GtkWidget *druid)
-{
 	gnome_druid_set_page (GNOME_DRUID (druid),
-			      GNOME_DRUID_PAGE (druid_data.less));
-	return TRUE;
-}
-
-gboolean
-on_core_page_back (GtkWidget *page, GtkWidget *druid)
-{
-	gnome_druid_set_page (GNOME_DRUID (druid),
-			      GNOME_DRUID_PAGE (druid_data.nature));
+			      GNOME_DRUID_PAGE (druid_data.action));
 	return TRUE;
 }
 
@@ -294,26 +302,6 @@ on_stop_button_clicked (GtkWidget *button, gpointer data)
 	}
 }
 
-gboolean
-on_less_page_back (GtkWidget *page, GtkWidget *druid)
-{
-	GtkWidget *newpage = NULL;
-	switch (druid_data.crash_type) {
-	case CRASH_DIALOG:
-		newpage = druid_data.attach;
-		break;
-	case CRASH_CORE:
-		newpage = druid_data.core;
-		break;
-	default:
-		g_assert_not_reached ();
-		break;
-	}
-	gnome_druid_set_page (GNOME_DRUID (druid),
-			      GNOME_DRUID_PAGE (newpage));
-	return TRUE;
-}
-
 gboolean
 on_action_page_back (GtkWidget *page, GtkWidget *druid)
 {
@@ -325,9 +313,43 @@ on_action_page_back (GtkWidget *page, GtkWidget *druid)
 	return TRUE;
 }
 
+static void
+on_about_button_clicked (GtkWidget *button, gpointer data)
+{
+	static GtkWidget *about, *href;
+	static const char *authors[] = {
+		"Jacob Berkman  <jberkman@andrew.cmu.edu>",
+		NULL
+	};
+
+	if (about) {
+		gdk_window_show (about->window);
+		gdk_window_raise (about->window);
+		return;
+	}
+		
+	about = gnome_about_new (_(PACKAGE), VERSION,
+				 _("Copyright (C) 1999 Jacob Berkman"),
+				 authors,
+				 _("The GNOME graphical bug reporting tool"),
+				 "bug-buddy.png");
+	gtk_signal_connect (GTK_OBJECT (about), "destroy",
+			    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
+			    &about);
+
+	href = gnome_href_new ("http://www.andrew.cmu.edu/~jberkman/bug-buddy/",
+			       _("The lame bug-buddy web page"));
+	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (about)->vbox),
+			    href, FALSE, FALSE, 0);
+
+	gtk_widget_show (href);
+	gtk_widget_show (about);
+}
+
 gboolean
 on_the_druid_cancel (GtkWidget *w)
 {
+	save_config ();
 	gtk_main_quit ();
 	return FALSE;
 }
@@ -348,6 +370,40 @@ on_contact_page_next (GtkWidget *page, GtkWidget *druid)
 	if (!s || strlen(s) == 0)
 		goto contact_failed;
 
+	w = glade_xml_get_widget (druid_data.xml, "mailer_entry");
+	s = gtk_entry_get_text (GTK_ENTRY (w));
+	if (!s || strlen (s) == 0 || !g_file_exists (s)) {
+		if (druid_data.submit_type == SUBMIT_TO_SELF ||
+		    druid_data.submit_type == SUBMIT_REPORT) {
+			GtkWidget *d;
+			char *m;
+			m = g_strdup_printf (_("'%s' doesn't seem to exist.  "
+					       "You won't be able to actually\n"
+					       "submit a but report, but you will "
+					       "be able to save it to a file.\n\n"
+					       "Specify a new location for sendmail?"),
+					     s);	
+			d = gnome_question_dialog (m, NULL, NULL);
+			g_free (m);
+			if (GNOME_YES == gnome_dialog_run_and_close (GNOME_DIALOG (d)))
+				return TRUE;
+		}
+			    
+		w = glade_xml_get_widget (druid_data.xml, "submit_radio");
+		gtk_widget_hide (w);
+		w = glade_xml_get_widget (druid_data.xml, "email_radio");
+		gtk_widget_hide (w);		
+		w = glade_xml_get_widget (druid_data.xml, "file_radio");
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+					      TRUE);
+		return FALSE;
+	}
+
+	w = glade_xml_get_widget (druid_data.xml, "submit_radio");
+	gtk_widget_show (w);
+	w = glade_xml_get_widget (druid_data.xml, "email_radio");
+	gtk_widget_show (w);
+
 	return FALSE;
 
  contact_failed:
@@ -362,21 +418,31 @@ on_complete_page_prepare (GtkWidget *page, GtkWidget *druid)
 	gchar *name, *from, *to, *package, *subject;
 	gchar *text;
 	GtkWidget *w;
-
+	int bugnum;
 	to = SUBMIT_ADDRESS;
 
 	w = glade_xml_get_widget (druid_data.xml, "email_entry");
-	from = gtk_entry_get_text (GTK_ENTRY (w));
+	from = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
+
+	to = NULL;
 
 	switch (druid_data.submit_type) {
 	case SUBMIT_TO_SELF:
-		to = from;
+		to = g_strdup (from);
 		break;
 	case SUBMIT_REPORT:
+		if (druid_data.bug_type == BUG_NEW) {
+			to = g_strdup ("submit" SUBMIT_ADDRESS);
+			break;
+		}
+
+		w = glade_xml_get_widget (druid_data.xml, "bug_number");
+		bugnum = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (w));
+		to = g_strdup_printf ("%d" SUBMIT_ADDRESS, bugnum);
 		break;
 	case SUBMIT_FILE:
 		w = glade_xml_get_widget (druid_data.xml, "file_entry");
-		to = gtk_entry_get_text (GTK_ENTRY (w));
+		to = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
 		break;
 	case SUBMIT_NONE:
 		gnome_druid_page_finish_set_text (GNOME_DRUID_PAGE_FINISH (page),
@@ -418,17 +484,62 @@ on_complete_page_prepare (GtkWidget *page, GtkWidget *druid)
 	return FALSE;
 }
 
+static void
+write_line_width (FILE *fp, char *s)
+{
+	gchar *sp;
+	if (!s)
+		return;
+
+	if (strlen (s) < LINE_WIDTH) {
+		fprintf (fp, "%s\n", s);
+		return;
+	}
+
+	for (sp = s+LINE_WIDTH; sp > s && !isspace (*sp); sp--)
+		;
+
+	if (s == sp)
+		sp = strpbrk (s+LINE_WIDTH, "\t\n ");
+       
+	if (sp)
+		*sp = '\0';
+
+	fprintf (fp, "%s\n", s);
+
+	if (sp)
+		write_line_width (fp, sp+1);
+}
+
+static void
+write_line_widthv (FILE *fp, const char *s)
+{
+	int i;
+	gchar **sv = g_strsplit (s, "\n", 0);
+	
+	for (i = 0; sv[i]; i++)
+		write_line_width (fp, sv[i]);
+	
+	g_strfreev (sv);
+}
+
 gboolean
 on_complete_page_finish (GtkWidget *page, GtkWidget *druid)
 {
 	GtkWidget *w;
 	gchar *s, *s2, *s3, *subject;
+	char *command;
 	FILE *fp = stdout;
-	ListData *data;	
-	int status;
-	
-	s2 = SUBMIT_ADDRESS;
+	int status, bugnum, i;
 
+	if (druid_data.bug_type == BUG_NEW) {
+		s2 = g_strdup ("submit" SUBMIT_ADDRESS);
+	} else {	
+		w = glade_xml_get_widget (druid_data.xml, "bug_number");
+		bugnum = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (w));
+		s2 = g_strdup_printf ("%d" SUBMIT_ADDRESS, bugnum);
+	}
+		
 	w = glade_xml_get_widget (druid_data.xml, "email_entry");
 	s = gtk_entry_get_text (GTK_ENTRY (w));
 
@@ -437,17 +548,21 @@ on_complete_page_finish (GtkWidget *page, GtkWidget *druid)
 		s2 = s;
 		/* fall through */
 	case SUBMIT_REPORT:
-		/*s3 = g_strconcat (druid_data.mail_cmd, s2, NULL);*/
-		g_message (_("about to run '%s'"), druid_data.mail_cmd);
-		fp = popen (druid_data.mail_cmd, "w");
+		w = glade_xml_get_widget (druid_data.xml, "mailer_entry");
+		s3 = gtk_entry_get_text (GTK_ENTRY (w));
+		command = g_strconcat (s3, " -i -t", NULL);
+		g_message (_("about to run '%s'"), command);
+		fp = popen (command, "w");	       
 		if (!fp) {
 			s = g_strdup_printf (_("Unable to start mail program:\n"
-					       "'%s'"), druid_data.mail_cmd);
+					       "'%s'"), s3);
 			w = gnome_error_dialog (s);
 			g_free (s);
+			g_free (command);
 			gnome_dialog_run_and_close (GNOME_DIALOG (w));
 			return FALSE;
 		}
+		g_free (command);
 		break;
 	case SUBMIT_FILE:
 		w = glade_xml_get_widget (druid_data.xml, "file_entry");
@@ -471,7 +586,7 @@ on_complete_page_finish (GtkWidget *page, GtkWidget *druid)
 		return FALSE;
 	}
 
-	fprintf (fp, "From %s\nTo: %s\n", g_get_user_name(), s2);
+	fprintf (fp, "To: %s\n", s2);
 
 	w = glade_xml_get_widget (druid_data.xml, "name_entry");
 	s2 = gtk_entry_get_text (GTK_ENTRY (w));
@@ -496,30 +611,38 @@ on_complete_page_finish (GtkWidget *page, GtkWidget *druid)
 		 ">Synopsis: %s\n"
 		 ">Class: %s\n", s, subject, bug_class[druid_data.bug_class][1]);
 
-	for (data  = list_data; data->label; data++) {
-		s = NULL;
+	for (i = 0; i < GTK_CLIST (druid_data.version_list)->rows; i++) {
+		s = s2 = NULL;
 		gtk_clist_get_text (GTK_CLIST (druid_data.version_list),
-				    data->row, 1, &s);
-		if (s && strlen (s))
-			fprintf (fp, "%s: %s\n", data->label, s);
+				    i, 0, &s);
+		gtk_clist_get_text (GTK_CLIST (druid_data.version_list),
+				    i, 1, &s2);
+		
+		if (s && strlen (s) &&
+		    s2 && strlen (s2))
+			fprintf (fp, "%s: %s\n", s, s2);
 	}
 
 	w = glade_xml_get_widget (druid_data.xml, "desc_area");
 	s = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
-	if (s && strlen (s))
-		fprintf (fp, "\n>Description:\n%s\n", s);
+	if (s && strlen (s)) {
+		fprintf (fp, "\n\n>Description:\n");
+		write_line_widthv (fp, s);
+	}
 	g_free (s);
 
 	w = glade_xml_get_widget (druid_data.xml, "repeat_area");
 	s = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
-	if (s && strlen(s))
-		fprintf (fp, "\n>How-To-Repeat:\n%s\n", s);
+	if (s && strlen(s)) {
+		fprintf (fp, "\n\n>How-To-Repeat:\n");
+		write_line_widthv (fp, s);
+	}
 	g_free (s);
 
 	w = glade_xml_get_widget (druid_data.xml, "gdb_text");
 	s = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
 	if (s && strlen(s))
-		fprintf (fp, "\nDebugging information:\n%s\n", s);
+		fprintf (fp, "\n\nDebugging information:\n%s\n", s);
 	g_free (s);
 
 	status = pclose (fp);
@@ -537,7 +660,7 @@ on_version_list_select_row (GtkCList *list, gint row, gint col,
 			    GdkEventButton *event, gpointer udata)
 {	
 	gchar *s;
-	druid_data.selected_data = gtk_clist_get_row_data (list, row);
+	druid_data.selected_row = row;
 	if (gtk_clist_get_text (list, row, 1, &s))
 		gtk_entry_set_text (GTK_ENTRY (druid_data.version_edit), s);
 	else
@@ -552,9 +675,9 @@ update_selected_row (GtkWidget *w, gpointer data)
 {
 	gchar *s;
 	gint row;
-	if (!druid_data.selected_data)
+	if (druid_data.selected_row == -1)
 		return;
-	row = druid_data.selected_data->row;
+	row = druid_data.selected_row;
 	s = gtk_entry_get_text (GTK_ENTRY (druid_data.version_edit));
 	gtk_clist_set_text (GTK_CLIST (druid_data.version_list), row, 1, s);
 }
@@ -602,31 +725,35 @@ on_action_page_next (GtkWidget *page, GtkWidget *druid)
 	return (GNOME_NO == gnome_dialog_run_and_close (GNOME_DIALOG (d)));
 }
 
-static gchar *
-get_data_from_command (const gchar *cmd)
+void
+on_newreport_radio_toggled (GtkWidget *w, gpointer data)
 {
-	static gchar buf[1024];
-	int status;
-	FILE *fp;
+	GtkWidget *w2;
+	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)))
+		return;
 
-	fp = popen (cmd, "r");
+	w2 = glade_xml_get_widget (druid_data.xml, "newbug_table");
+	gtk_widget_show (w2);
 
-	if (!fp) {
-		g_message (_("Could not run command '%s'"), cmd);
-		return NULL;
-	}
+	w2 = glade_xml_get_widget (druid_data.xml, "existing_box");
+	gtk_widget_hide (w2);
 
-	if (!fgets (buf, 1024, fp)) {
-		g_message (_("Error reading input of '%s'"), cmd);
-		return NULL;
-	} 
+	druid_data.bug_type = BUG_NEW;
+}
+void
+on_existing_radio_toggled (GtkWidget *w, gpointer data)
+{
+	GtkWidget *w2;
+	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)))
+		return;
 
-	status = pclose (fp);
-#if 0
-	g_message (_("Subprocess exited with status %d"), status);
-#endif
+	w2 = glade_xml_get_widget (druid_data.xml, "newbug_table");
+	gtk_widget_hide (w2);
 
-	return g_strchomp (buf);	
+	w2 = glade_xml_get_widget (druid_data.xml, "existing_box");
+	gtk_widget_show (w2);
+
+	druid_data.bug_type = BUG_EXISTING;
 }
 
 static gboolean
@@ -643,16 +770,15 @@ set_bug_class (GtkWidget *w, gpointer data)
 	return FALSE;
 }
 
+
 GtkWidget *
 make_pixmap_button (gchar *widget_name, gchar *text, 
-		    gchar *s2, int stop, int i2)
+		    gchar *s2, int type, int i2)
 {
 	GtkWidget *p, *w;
-	char *type = stop 
-		? GNOME_STOCK_PIXMAP_STOP
-		: GNOME_STOCK_PIXMAP_REFRESH;
 
-	p = gnome_stock_pixmap_widget (NULL, type);
+	/* this is slightly bad but should be ok */
+	p = gnome_stock_pixmap_widget (NULL, text);
 	w = gnome_pixmap_button (p, text);
 	GTK_WIDGET_SET_FLAGS (w, GTK_CAN_DEFAULT);
 	gtk_container_set_border_width (GTK_CONTAINER (w), GNOME_PAD_SMALL);
@@ -674,14 +800,6 @@ make_anim (gchar *widget_name, gchar *imgname,
 	return druid_data.gdb_anim;
 }
 
-static gboolean
-clean_hash (gpointer key, gpointer value, gpointer data)
-{
-	g_free (key);
-	g_free (value);
-	return TRUE;
-}
-
 /* ugly stupid dumb stuff stolen from the bug reporting web page */
 static gchar *
 get_package_from_appname (const char *appname)
@@ -719,8 +837,7 @@ get_package_from_appname (const char *appname)
 	package = g_strdup (g_hash_table_lookup (table,
 						 g_filename_pointer (appname)));
 	
-	g_hash_table_foreach_remove (table, clean_hash, NULL);
-	g_hash_table_destroy (table);
+	destroy_hash_table (table, TRUE);
 
 	return package;
 }
@@ -740,15 +857,14 @@ init_toggle (const char *name, int test, int data, GtkSignalFunc func)
 static void
 init_ui ()
 {
-	GtkWidget *w, *m;
-	ListData *data;
+	GtkWidget *w, *m, *segv, *core;
 	gchar *row[3] = { NULL };
-	gchar *s, *package = NULL;
+	gchar *s, *p = NULL;
 	int i;
 
 	glade_xml_signal_autoconnect (druid_data.xml);
 
-	load_config ();
+	load_config ();	
 
 	w = glade_xml_get_widget (druid_data.xml, "the_druid");
 	druid_data.the_druid = w;
@@ -801,17 +917,17 @@ init_ui ()
 	if (s)
 		gtk_entry_set_text (GTK_ENTRY (w), s);
 
-	package = g_strdup (popt_data.package);
-	if (!package && s)
-		package = get_package_from_appname (s);
+	p = g_strdup (popt_data.package);
+	if (!p && s)
+		p = get_package_from_appname (s);
 
-	if (!package)
-		package = g_strdup ("general");
+	if (!p)
+		p = g_strdup ("general");
 
 	w = glade_xml_get_widget (druid_data.xml, 
 				  "package_entry");
-	gtk_entry_set_text (GTK_ENTRY (w), package);
-	g_free (package);
+	gtk_entry_set_text (GTK_ENTRY (w), p);
+	g_free (p);
 
 	w = druid_data.pid =
 		glade_xml_get_widget (druid_data.xml, "crashed_pid");
@@ -865,12 +981,23 @@ init_ui ()
 	init_toggle ("file_radio", druid_data.submit_type, SUBMIT_FILE,
 		     GTK_SIGNAL_FUNC (update_submit_type));
 
-	/* text areas */
-	w = glade_xml_get_widget (druid_data.xml, "desc_area");
-	gtk_text_set_line_wrap (GTK_TEXT (w), FALSE);
+	segv = glade_xml_get_widget (druid_data.xml, "segv_table");
+	core = glade_xml_get_widget (druid_data.xml, "core_box");
 
-	w = glade_xml_get_widget (druid_data.xml, "repeat_area");
-	gtk_text_set_line_wrap (GTK_TEXT (w), FALSE);
+	switch (druid_data.crash_type) {
+	case CRASH_DIALOG:
+		gtk_widget_show (segv);
+		gtk_widget_hide (core);
+		break;
+	case CRASH_CORE:
+		gtk_widget_show (core);
+		gtk_widget_hide (segv);
+		break;
+	case CRASH_NONE:
+		gtk_widget_hide (core);
+		gtk_widget_hide (segv);
+		break;
+	}
 
 	/* system config page */
 	druid_data.version_edit =
@@ -879,15 +1006,42 @@ init_ui ()
 		glade_xml_get_widget (druid_data.xml, "version_label");
 	druid_data.version_list = w =
 		glade_xml_get_widget (druid_data.xml, "version_list");
-	for (data = list_data; data->label; data++) {
-		for (i = 0; data->cmds[i] && !row[1]; i++)
-			row[1] = get_data_from_command (data->cmds[i]);
-		row[0] = _(data->label);
-		data->row = gtk_clist_append (GTK_CLIST (w), row);
-		gtk_clist_set_row_data (GTK_CLIST (w), data->row, data);
-		row[1] = NULL;
+
+	for (i = 0; distros[i].name; i++) {
+		if (!g_file_exists (distros[i].version_file))
+			continue;
+		row[0] = _("Distribution");
+		row[1] = distros[i].phylum->version (&distros[i]);
+		gtk_clist_append (GTK_CLIST (w), row);
+		druid_data.distro = &distros[i];
+		break;
 	}
 
+	/* so we have like 3 passages, but it is better */
+	for (i = 0; package[i].name; i++) {
+		if (!package[i].pre_command)
+			continue;
+		package[i].version = get_line_from_command (package[i].pre_command);
+	}
+
+	if (druid_data.distro)
+		druid_data.distro->phylum->packager (package);
+
+	for (i = 0; package[i].name; i++) {
+		if (!package[i].version &&
+		    package[i].post_command)
+			package[i].version = 
+				get_line_from_command (package[i].post_command);
+		row[0] = _(package[i].name);
+		row[1] = package[i].version;
+		gtk_clist_append (GTK_CLIST (w), row);
+	}
+
+	w = glade_xml_get_widget (druid_data.xml, "about_button");
+	gtk_signal_connect (GTK_OBJECT (w), "clicked",
+			    GTK_SIGNAL_FUNC (on_about_button_clicked),
+			    NULL);
+
 	/* less page */
 	druid_data.stop_button = w = 
 		glade_xml_get_widget (druid_data.xml, "stop_button");
@@ -909,12 +1063,12 @@ init_ui ()
 						    "nature_page");
 	druid_data.attach   = glade_xml_get_widget (druid_data.xml,
 						    "attach_page");
-	druid_data.core   = glade_xml_get_widget (druid_data.xml,
-						  "core_page");
-	druid_data.less   = glade_xml_get_widget (druid_data.xml, 
-						  "less_page");
-	druid_data.action = glade_xml_get_widget (druid_data.xml, 
-						  "action_page");
+	druid_data.core     = glade_xml_get_widget (druid_data.xml,
+						    "core_page");
+	druid_data.less     = glade_xml_get_widget (druid_data.xml, 
+						    "less_page");
+	druid_data.action   = glade_xml_get_widget (druid_data.xml, 
+						    "action_page");	
 }
 
 int
@@ -933,20 +1087,6 @@ main (int argc, char *argv[])
 				    options, 0, NULL);
 	glade_gnome_init ();
 
-
-	/* from gnome-bug */
-	if (g_file_exists ("/usr/sbin/sendmail"))
-		druid_data.mail_cmd = "/usr/sbin/sendmail -t";
-	else if (g_file_exists ("/usr/lib/sendmail"))
-		druid_data.mail_cmd = "/usr/lib/sendmail -t";
-	else if ( !(druid_data.mail_cmd = 
-		    gnome_is_program_in_path ("rmail")) ) {
-		GtkWidget *d = gnome_error_dialog (_("Could not find a mail program.\n"));
-		gnome_dialog_run_and_close (GNOME_DIALOG (d));
-		return 0;
-	}
-
-
 	xml_path = BUDDY_DATADIR "/bug-buddy.glade";
 	if (!g_file_exists (xml_path)) {
 		s = g_strdup_printf (_("Could not find our glade file '%s'\n"
