@@ -24,15 +24,9 @@
 
 #include <glade/glade.h>
 #include <sys/types.h>
+#include "distro.h"
 
-#define SUBMIT_ADDRESS "submit@bugs.gnome.org";
-#define COMMAND_SIZE 5
-
-typedef struct {
-	const gchar *label;
-	const gchar *cmds[COMMAND_SIZE];	
-	gint row;
-} ListData;
+#define SUBMIT_ADDRESS "@bugs.gnome.org"
 
 typedef enum {
 	CRASH_NONE,
@@ -47,6 +41,11 @@ typedef enum {
 	SUBMIT_FILE
 } SubmitType;
 
+typedef enum {
+	BUG_NEW,
+	BUG_EXISTING
+} BugType;
+
 typedef struct {
 	GtkWidget *the_druid;
 
@@ -65,14 +64,16 @@ typedef struct {
 	GtkWidget *version_edit;
 	GtkWidget *version_label;
 	GtkWidget *version_list;
-	ListData *selected_data;
+	int selected_row;
+	Distribution *distro;
 
 	GtkWidget *stop_button;
 	GtkWidget *refresh_button;
 
-	gchar *mail_cmd;
 	CrashType crash_type;
 	SubmitType submit_type;
+	BugType bug_type;
+
 	int severity;
 	int bug_class;
 
